﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Windows.Threading;

namespace SampleForSupport
{

    /// <summary>
    /// Classe de Base pour les modèles view de l'application.
    /// Intègre les méthodes de gestion des pages.
    /// </summary>
    public abstract class ViewModelBase 
    {


        /// <summary>
        /// Evènement de changement d'un propriété
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;



        #region Surcharge NotifyPRopertyChanged

        /// <summary>
        /// Notify using pre-made PropertyChangedEventArgs
        /// </summary>
        /// <param name="args">Argument de l'évènement</param>
        protected new void NotifyPropertyChanged(PropertyChangedEventArgs args)
        {
            PropertyChangedEventHandler handler = PropertyChanged;

            if (handler != null)
            {
                Dispatcher dispatcher =
                    (from PropertyChangedEventHandler nh in handler.GetInvocationList()
                     let dpo = nh.Target as DispatcherObject
                     where dpo != null
                     select dpo.Dispatcher).FirstOrDefault();

                if (dispatcher != null && dispatcher.CheckAccess() == false)
                {
                    dispatcher.Invoke(DispatcherPriority.DataBind,
                        (Action)(delegate()
                        {
                            NotifyPropertyChanged(args);
                            //demande au dispactcher (main thread) de réévaluer les commandes des boutons
                            System.Windows.Input.CommandManager.InvalidateRequerySuggested();
                        }));
                }
                else
                {
                    foreach (PropertyChangedEventHandler nh
                            in handler.GetInvocationList())
                        nh.Invoke(this, args);
                }
            }
        }

        /// <summary>
        /// Notify using String property name
        /// </summary>
        /// <param name="propertyName">Nom de la propriété ayant changé</param>
        protected new void NotifyPropertyChanged(String propertyName)
        {
            this.NotifyPropertyChanged(new PropertyChangedEventArgs(propertyName));
        }

        #endregion
    }
}
